#include <LiquidCrystal.h>
#include <HX711_ADC.h>

// Define HX711 pins
#define DOUT_PIN 2
#define SCK_PIN 3

// Initialize the HX711
HX711_ADC LoadCell(DOUT_PIN, SCK_PIN);

// Initialize the LCD library with the numbers of the interface pins
const int rs = 13, en = 12, d4 = 4, d5 = 5, d6 = 6, d7 = 7;
LiquidCrystal lcd(rs, en, d4, d5, d6, d7);

const int buttonPin = 8;  // Define the pin for the button
const int ledPin = 9; 
const int plateButton= 10;

void setup() {
  // initialize the LED pin as an output:
  pinMode(ledPin, OUTPUT);
  // initialize the pushbutton pin as an input:
  pinMode(buttonPin, INPUT);
  pinMode(plateButton, INPUT);
  
  Serial.begin(57600);

  // Start the connection to HX711
  LoadCell.begin();
  LoadCell.start(1000); // Allow the load cells to stabilize for 1000ms
  LoadCell.setCalFactor(-386.62); // Set the calibration factor
  // -386.92
  // -385.88
  LoadCell.tare(); // Reset the scale to zero

  // Initialize the LCD
  lcd.begin(16, 2);
  lcd.setCursor(1, 0); // Set cursor to the first row
  lcd.print("Cook Pro"); // Print out to LCD
  lcd.setCursor(4, 1); // Set cursor to the second row
  lcd.print("Measure"); // Print out to LCD
  delay(3000);
  lcd.clear();
}

void loop() {
  static float lastWeight = 0;
  static bool isStable = false;
  static bool isFirstRun = true; // Add a variable to track the first run
  static bool plate = false;
  static float plateWeight = 0;
  LoadCell.update();
  float weight = LoadCell.getData();

  Serial.print("Weight: ");
  Serial.print(weight, 2); // Print the weight with 2 decimal places
  Serial.println(" g");

  // Retrieve data from the load cell
  if (isFirstRun) {
    lastWeight = 0.0; // Set lastWeight to 0.0 for the first run
    isFirstRun = false; // Set isFirstRun to false after the first run
  }

  // Check if weight is stable
  if (abs(weight - lastWeight) < 0.05) { // Adjust the threshold as needed
    isStable = true;
  } else {
    isStable = false;
  }

  if (weight < 0) {
    weight = 0;
  } else if (weight > 0 && weight < 1) {
    weight = 0;
  }

  if (isStable) {
    if (plate == false) {
      lcd.clear();
      lcd.setCursor(0, 0); // Set cursor to the first row
      lcd.print("Weight: ");
      lcd.print(weight); // Print the weight with 2 decimal places
      lcd.print(" g");
    } else {
      if (weight < 20 && weight > 15) {
        plateWeight = weight;
        weight = 0;
      }
      if (weight > plateWeight) {
        weight = weight - plateWeight;
      }
      lcd.clear();
      lcd.setCursor(0, 0); // Set cursor to the first row
      lcd.print("Weight: ");
      lcd.print(weight); // Print the weight with 2 decimal places
      lcd.print(" g");

      lcd.setCursor(0, 1); // Adjust cursor position
      lcd.print("With plate");
    }
  }

  lastWeight = weight;

  // Check if the button is pressed
  if (digitalRead(buttonPin) == HIGH) {
    digitalWrite(ledPin, LOW);
    Serial.println("Reset button pressed");
    plate = false;
    weight = 0;
    lcd.clear();
    lcd.setCursor(0, 1); // set cursor to second row
    lcd.print("    Reset...    ");
    LoadCell.start(1000);
    LoadCell.tare(); // Reset the scale to zero
    delay(1000); // Allow some time for the reset to stabilize
    lcd.clear();
  }

  if (digitalRead(plateButton) == HIGH) {
    // turn LED on:
    Serial.println("PLATE FUNCTION");
    digitalWrite(ledPin, HIGH);
    plate = true;
    lcd.clear();
    lcd.setCursor(0, 1); // set cursor to second row
    lcd.print(" Plate function ");
    LoadCell.start(1000);
    LoadCell.tare(); // Reset the scale to zero
    delay(1000); // Allow some time for the reset to stabilize
    lcd.clear();
  }

  delay(100); // Update every 100ms
}
